@@ -1,10 +1,11 @@
-from .crypto_functions import (toBase, fromBase, StringToNum, NumToString, ExtendedGCD, makeChineseRemainder,
+from .crypto_functions import (toBase, fromBase, StringToNum, NumToString, ExtendedGCD,
                                ChineseRemainder, baby_step_giant_step, pohlig_hellman, DSA,
                                pollard_p1, FactorInt, index_calculus_dlp, StringToElliptic, EllipticToString, Elliptic,
-                               EllipticCurve, SolveDLP, elliptic_bsgs, calculate_state)
+                               EllipticCurve, pollard_rho_dlp, elliptic_bsgs, calculate_state, sqrt_safe)
+from .quadratic_sieve import quadratic_sieve, exp_value, factor_if_smooth
 from .bailliepsw_helper import Jacobi
 from .prime import IsPrime, MillerRabin_bases, RandomPrime, ConfirmPrime, NextPrime, PrevPrime, BailliePSW_Primality
-from .linear_algebra import Matrix, LinearMap, where, aslist, isnumber, all_elements
+from .linear_algebra import Matrix, LinearMap, where, aslist, isnumber, all_elements, python_number, is_binary_matrix
 import setuptools.version
 
 
